-------------------------------------------------------------------------------
--
-- VGA/SRAM supercontroller
--  This module integrates the VGA and SRAM modules
-- Written for 128-bit AES decryption project
-- Course: CSEE 4840 - Embedded System Design, Spring 2008
-- Authors: Shrivathsa Bhargav (sb2784)
--			Larry Chen (lc2454)
--			Abhinandan Majumdar (am2993)
--			Shiva Ramudit (syr9)
--
-- Last modified: 5-8-2008
-------------------------------------------------------------------------------


library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity vga_sram_supercontroller is
  
  port (
    reset : in std_logic;
    CLOCK_50 : in std_logic;

    signal chipselect : in std_logic;
    signal write, read : in std_logic;
    signal address  :  in std_logic_vector(20 downto 0);
    signal readdata : out std_logic_vector(15 downto 0);
    signal writedata : in std_logic_vector(15 downto 0);
    signal byteenable : in std_logic_vector(1 downto 0);

    VGA_CLK,                         		  -- Clock
    VGA_HS,                          		  -- H_SYNC
    VGA_VS,                          		  -- V_SYNC
    VGA_BLANK,                       		  -- BLANK
    VGA_SYNC : out std_logic;       		  -- SYNC
    VGA_R,
    VGA_G,
    VGA_B : out std_logic_vector(9 downto 0); -- R=G=B[9:0]

    SRAM_DQ   : inout std_logic_vector(15 downto 0);
    SRAM_ADDR : out std_logic_vector(17 downto 0);
    SRAM_UB_N, SRAM_LB_N : out std_logic;
    SRAM_WE_N, SRAM_CE_N : out std_logic;
    SRAM_OE_N : out std_logic
    );

end vga_sram_supercontroller;

architecture vga_sram_supercontroller_arch of vga_sram_supercontroller is

  signal address2 : std_logic_vector(17 downto 0) := "000000000000110010";
  signal readdata2 : std_logic_vector(15 downto 0);
  signal go	: std_logic := '1';
  signal clk25 : std_logic := '0';	--25 MHz clock for the VGA controller
  signal clock_div : unsigned(25 downto 0) := "00000000000000000000000000";
  signal rectangle_from_vga : std_logic := '0';
  signal readdata_from_sram : std_logic_vector(15 downto 0);

component de2_sram_controller is
  
  port (
    signal chipselect : in std_logic;
    signal write, read : in std_logic;
    signal address  :  in std_logic_vector(17 downto 0);
    signal readdata : out std_logic_vector(15 downto 0);
    signal writedata : in std_logic_vector(15 downto 0);
    signal byteenable : in std_logic_vector(1 downto 0);

	--From VGA
    signal address2 :  in std_logic_vector(17 downto 0);
    signal readdata2 : out std_logic_vector(15 downto 0);

    signal go : in std_logic;		--Go from Nios

    signal SRAM_DQ : inout std_logic_vector(15 downto 0);
    signal SRAM_ADDR : out std_logic_vector(17 downto 0);
    signal SRAM_UB_N, SRAM_LB_N : out std_logic;
    signal SRAM_WE_N, SRAM_CE_N : out std_logic;
    signal SRAM_OE_N : out std_logic
    );

end component de2_sram_controller;


component de2_vga_raster is
  
  port (
--    reset : in std_logic;
    clk   : in std_logic;      -- Should be 25.125 MHz

    -- SRAM signals
    address_out : out std_logic_vector(17 downto 0);
    data_in		: in std_logic_vector(15 downto 0);

    -- Rectangle
    rectangle_out : out std_logic;

    VGA_CLK,                         -- Clock
    VGA_HS,                          -- H_SYNC
    VGA_VS,                          -- V_SYNC
    VGA_BLANK,                       -- BLANK
    VGA_SYNC : out std_logic;        -- SYNC
    VGA_R,
    VGA_G,
    VGA_B : out std_logic_vector(9 downto 0) -- R=G=B[9:0]
    );

end component de2_vga_raster;
begin

  --Generate a 25 MHz clock for the VGA (clock divider)
  process (CLOCK_50)
  begin
    if rising_edge(CLOCK_50) then
        clk25 <= not clk25;
    end if;
  end process;


vga: de2_vga_raster port map (
    clk => clk25,
    address_out => address2,
    data_in => readdata2,
    rectangle_out => rectangle_from_vga,
    VGA_CLK => VGA_CLK,
    VGA_HS => VGA_HS,
    VGA_VS => VGA_VS,
    VGA_BLANK => VGA_BLANK,
    VGA_SYNC => VGA_SYNC,
    VGA_R => VGA_R,
    VGA_G => VGA_G,
    VGA_B => VGA_B
);

sram: de2_sram_controller port map (
    read => read,
    chipselect => chipselect,
    write => write,
    address  => address(17 downto 0),
    readdata => readdata_from_sram,
    writedata => writedata,
    byteenable => byteenable,
    address2 => address2,
    readdata2 => readdata2,
    
    go => go,

    SRAM_DQ => SRAM_DQ,
    SRAM_ADDR => SRAM_ADDR,
    SRAM_UB_N => SRAM_UB_N,
    SRAM_LB_N => SRAM_LB_N,
    SRAM_WE_N => SRAM_WE_N,
    SRAM_CE_N => SRAM_CE_N,
    SRAM_OE_N => SRAM_OE_N
  );

  --Check for go signal
  process(CLOCK_50)
  begin
  if rising_edge(CLOCK_50) then
    if address(19) = '1' then
      if write = '1' then
        go <= writedata(0);
      end if;
    end if;
    if address(20) = '1' then   --Are we inside the rectangle?
      if read = '1' then
        readdata <= "000000000000000" & rectangle_from_vga;
      end if;
    end if;
  end if;
  end process;

end vga_sram_supercontroller_arch;